//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import KakaoPartnerSDKCommon
import KakaoSDKCommon
import RxKakaoSDKCommon
import KakaoSDKAuth
import RxKakaoSDKAuth


@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {

    var window: UIWindow?
    var rxSampleVC: RxSDKSampleViewController!
    var rxPartnerSampleVC: RxPartnerSDKSampleViewController!
//    var sampleVC: SDKSampleViewController!

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        
        // Override point for customization after application launch.
        UITabBarController.setPhase(phase: Properties.phase())
        
        // Override point for customization after application launch.
        window = UIWindow(frame: UIScreen.main.bounds)
        
        let tabBarController = UITabBarController()
        let systemFontAttributes = [NSAttributedString.Key.font: UIFont.systemFont(ofSize: 14.0)]
        UITabBarItem.appearance().setTitleTextAttributes(systemFontAttributes, for: .normal)
        
        self.rxSampleVC = RxSDKSampleViewController()
        self.rxSampleVC.navigationItem.rightBarButtonItems = [UIBarButtonItem(title: "Log", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.showLog)),
                                                            UIBarButtonItem(title: "Phase", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.changePhase))]
        self.rxSampleVC.setTitle(args:Properties.phase()?.rawValue)
        
        let firstNaviController = UINavigationController(rootViewController:self.rxSampleVC)
        firstNaviController.tabBarItem = UITabBarItem(title: "RxKakaoSDK", image:nil, tag: 0)
        
        self.rxPartnerSampleVC = RxPartnerSDKSampleViewController()
        self.rxPartnerSampleVC.navigationItem.rightBarButtonItems = [UIBarButtonItem(title: "Log", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.showLog)),
                                                       UIBarButtonItem(title: "Phase", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.changePhase))]
        self.rxPartnerSampleVC.setTitle(args:Properties.phase()?.rawValue)
        
        let secondNaviController = UINavigationController(rootViewController:self.rxPartnerSampleVC)
        secondNaviController.tabBarItem = UITabBarItem(title: "RxKakaoPartnerSDK", image:nil, tag: 1)
        
//        self.sampleVC = SDKSampleViewController()
//        self.sampleVC.navigationItem.rightBarButtonItems = [UIBarButtonItem(title: "Log", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.showLog)),
//                                                       UIBarButtonItem(title: "Phase", style: UIBarButtonItem.Style.plain, target: tabBarController, action: #selector(tabBarController.changePhase))]
//        self.sampleVC.setTitle(args:Properties.phase()?.rawValue)
//
//        let thirdNaviController = UINavigationController(rootViewController:self.sampleVC)
//        thirdNaviController.tabBarItem = UITabBarItem(title: "KakaoSDK", image:nil, tag: 2)
//        tabBarController.viewControllers = [firstNaviController, secondNaviController, thirdNaviController]
        
        tabBarController.viewControllers = [firstNaviController, secondNaviController]
        tabBarController.view.backgroundColor = UIColor.green

        window?.rootViewController = tabBarController
        window?.makeKeyAndVisible()
        
        NotificationCenter.default.addObserver(self, selector: #selector(hostChanged), name: Notification.Name("HostChanged"), object: nil)
        
        return true
    }

    func applicationWillResignActive(_ application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
    }

    func applicationDidEnterBackground(_ application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    }

    func applicationWillEnterForeground(_ application: UIApplication) {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
    }

    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }

    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
        
        NotificationCenter.default.removeObserver(self, name: Notification.Name("HostChanged"), object: nil)
    }
    
    func application(_ app: UIApplication, open url: URL, options: [UIApplication.OpenURLOptionsKey : Any] = [:]) -> Bool {
        if (AuthApi.isKakaoTalkLoginUrl(url)) {
            return AuthController.rx.handleOpenUrl(url: url, options: options)
        }
        return false
    }
    
    @objc func hostChanged() {
        self.rxSampleVC.resetResultIndicator()
        self.rxSampleVC.setTitle(args:Properties.phase()?.rawValue)
        
        self.rxPartnerSampleVC.resetResultIndicator()
        self.rxPartnerSampleVC.setTitle(args:Properties.phase()?.rawValue)
    }
}

extension AppDelegate {    
    static func templateId() -> (message:Int64, link:Int64) {
        switch Properties.phase() {
        case .Dev:
            return (message:151202, link:151203)
        case .Sandbox:
            return (message:682, link:683)
        case .Cbt:
            fallthrough
        case .Production:
            return (message:19383, link:19387)
        default:
            return (message:19383, link:19387)
        }
    }
    
    static func calendarId() -> (calendar:String, event:String) {
        switch Properties.phase() {
        case .Dev:
            return (calendar:"subscribe_6374ca625941ee699a32165c", event:"639964bc7edd5d0c9b722c14")
        case .Sandbox:
            return (calendar:"subscribe_6374ca625941ee699a32165c", event:"639176efca3adc72adfff948")
        case .Cbt:
            fallthrough
        case .Production:
            return (calendar:"subscribe_6374ca625941ee699a32165c", event:"63996425afcec577cce94f0b")
        default:
            return (calendar:"subscribe_6374ca625941ee699a32165c", event:"63996425afcec577cce94f0b")
        }
    }
    
    static func settleId() -> String {
        switch Properties.phase() {
        case .Dev:
            return "" //alpha phase는 없다고 함.
        case .Sandbox:
            return "ee82f111-12f3-4a96-9bcf-8ff0e1f56cde"
        case .Cbt:
            fallthrough
        case .Production:
            return "f3318663-771a-4b24-8714-3f3061fa17cd"
        default:
            return "f3318663-771a-4b24-8714-3f3061fa17cd"
        }
    }
}

extension UITabBarController {
    
    override open var canBecomeFirstResponder: Bool {        
        return true
    }
    
    override open func motionBegan(_ motion: UIEvent.EventSubtype, with event: UIEvent?) {
        super.motionBegan(motion, with: event)
        
        if (motion == .motionShake) {
            print("shake was detected")
            self.showLog()
        }
    }
    
    @objc public func showLog() {
        let window = UIApplication.shared.keyWindow!
        
        let debugVC = SDKDebugViewController.shared
        let debugNaviController = SDKDebugNavigationController(rootViewController:debugVC)
        debugNaviController.modalPresentationStyle = .overCurrentContext
        debugNaviController.modalTransitionStyle = .crossDissolve
        

        if let modalViewController = window.rootViewController?.presentedViewController {
            if (modalViewController is SDKDebugNavigationController) {
                modalViewController.dismiss(animated: false, completion: nil)
            }
            else {
                modalViewController.present(debugNaviController, animated: false) {
                    debugVC.showLog()
                }
            }
        }
        else {
            window.rootViewController!.present(debugNaviController, animated: false) {
                debugVC.showLog()
            }
        }
    }
    
    @objc func changePhase(sender: Any) {
        var prevPhase : Phase = .Production
        if let storedPhase = Properties.phase() {
            prevPhase = storedPhase
        }
        
        let alert = UIAlertController(title: "Change Phase", message: "current:(\(prevPhase.rawValue))\nYou can change host", preferredStyle: .actionSheet)
        alert.addAction(UIAlertAction(title: "Dev", style:((prevPhase == .Dev) ? .destructive : .default) , handler:{ (UIAlertAction) in
            guard (prevPhase != .Dev) else { return }
            UITabBarController.setPhase(phase: .Dev, needRefresh: true)
            print("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[ Phase: Dev ]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]")
        }))
        
        alert.addAction(UIAlertAction(title: "Sandbox", style:((prevPhase == .Sandbox) ? .destructive : .default) , handler:{ (UIAlertAction) in
            
            guard (prevPhase != .Sandbox) else { return }
            UITabBarController.setPhase(phase: .Sandbox, needRefresh: true)
            print("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[ Phase: Sandbox ]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]")
        }))
        
        alert.addAction(UIAlertAction(title: "Cbt", style:((prevPhase == .Cbt) ? .destructive : .default) , handler:{ (UIAlertAction) in
            guard (prevPhase != .Cbt) else { return }
            UITabBarController.setPhase(phase: .Cbt, needRefresh: true)
            print("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[ Phase: Cbt ]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]")
        }))
        alert.addAction(UIAlertAction(title: "Production", style:((prevPhase == .Production) ? .destructive : .default) , handler:{ (UIAlertAction) in
            guard (prevPhase != .Production) else { return }
            UITabBarController.setPhase(phase: .Production, needRefresh: true)
            print("[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[[ Phase: Production ]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]]")
        }))
        
        alert.addAction(UIAlertAction(title: "Close", style: .cancel, handler:{ (UIAlertAction) in
            print("User click Close button")
        }))
        
        self.present(alert, animated: true, completion: {
            print("completion block")
        })
    }
    
    static func setPhase(phase:Phase?, needRefresh:Bool = false) {        
        if let phase = phase {
            switch phase {
            case .Dev:
                RxKakaoSDK.initSDK(appKey: "25f3dfe2da25c29a406d8f55de1d7dbb", loggingEnable: true, hosts:Hosts(phase:phase))
            case .Sandbox:
                RxKakaoSDK.initSDK(appKey: "2e3bcf1444acdc03f6267db1b05bb03e", loggingEnable: true, hosts:Hosts(phase:phase))
            case .Cbt:
                RxKakaoSDK.initSDK(appKey: "9f9de684c354a72d2eb2a540a11441c2", loggingEnable: true, hosts:Hosts(phase:phase))
            case .Production:
                RxKakaoSDK.initSDK(appKey: "9f9de684c354a72d2eb2a540a11441c2", loggingEnable: true, hosts:Hosts(phase:phase))
            }
            if (needRefresh) {
                Properties.setPhase(phase)
                Auth.shared.tokenManager.deleteToken()
                NotificationCenter.default.post(name: Notification.Name("HostChanged"), object: nil)
            }
        }
        else {
            RxKakaoSDK.initSDK(appKey: "9f9de684c354a72d2eb2a540a11441c2", loggingEnable: true, hosts:Hosts(phase:.Production))
            Properties.setPhase(.Production)
        }
    }
}
